<?php
/**
 * @author dongbeta
 * @copyright Copyright @ 2010 Xii Project
 * @link http://xii.googlecode.com
 * @license	 http://www.opensource.org/licenses/bsd-license.php
 * @version $Id: Config.php 92 2010-05-17 06:27:29Z dongbeta $
 * 
 * 使用此组件，可以方便地存储和读取网站配置，且数据格式需要支持serialize。
 * 
 * 新增或者更新配置：
 * Yii::app()->config->set('name', 'value');
 * 读取配置：
 * Yii::app()->config->get('name', 'default');
 * 
 * 数据库结构如下
 * CREATE TABLE `xii_config` (
 *  `name` varchar(64) NOT NULL,
 *  `value` text NOT NULL,
 *  UNIQUE KEY `name` (`name`)
 * );
 */
class Config extends CApplicationComponent
{

	/**
	 * @var string 数据库表名称
	 */
	public $table = '{{config}}';

	/**
	 * @var string 缓存前缀
	 */
	public $cachePrefix = 'Xii.Components.Config.';

	/**
	 * @var int 缓存周期，-1代表不缓存，0代表永远不会失效
	 */
	public $cachingDuration = 0;

	/**
	 * @var array 存储已经读取过的配置文件
	 */
	private $_config = array();

	/**
	 * 读取配置项
	 * @param string $name 配置项名称
	 * @param mixed $default 如果不存在的话，则返回此默认值
	 */
	public function get($name, $default = null)
	{
		//先从 $this->_config 中读取
		if(isset($this->_config[$name]))
			return $this->_config[$name];
		//没有，则从缓存中寻找
		if($this->cachingDuration >= 0)
			$config = Yii::app()->getCache()->get($this->cachePrefix . $name);
		//没有，则从数据库提取
		if($config === false) {
			$config = $this->find($name);
			//数据库没有，则返回$default设置的默认值
			if(!$config instanceof ConfigRecord)
				return $default;
			else
				$config = $config->value;
		}
		//至此，通过各种途径获得了值，则设置缓存
		if($this->cachingDuration >= 0)
			Yii::app()->getCache()->set($this->cachePrefix . $name, $config);
		//设置字段 $this->_config
		$this->_config[$name] = $config;
		//返回值
		return $value;
	}
	
	/**
	 * 设置配置项
	 * @param string $name 配置项名称
	 * @param mixed $value 配置项的值
	 */
	public function set($name, $value)
	{
		//如果数据中有这项记录，则保存到数据库，没有则会新增
		if($this->exist($name))
			$this->update($name, $value);
		else
			$this->add($name, $value);
		//设置缓存
		if($this->cachingDuration >= 0)
			Yii::app()->getCache()->set($this->cachePrefix . $name, $value);
		//设置字段
		$this->_config[$name] = $value;
	}

	/**
	 * 更新数据库记录
	 * @param string $name 配置项名称
	 * @param mixed $value 配置项的值
	 */
	protected function update($name, $value)
	{
		$config = $this->find($name);
		if ($config instanceof ConfigRecord) {
			$config->value = $value;
			return $config->save();
		}
		return false;	
	}

	/**
	 * 增加数据库记录
	 * @param string $name 配置项名称
	 * @param mixed $value 配置项的值
	 */
	protected function add($name, $value)
	{
		$config = new ConfigRecord();
		$config->setAttributes(array('name'=>$name, 'value'=>$value));
		return $config->save();
	}

	/**
	 * 从数据库中读取
	 * @param string $name 配置项名称
	 */
	protected function find($name)
	{
		return ConfigRecord::model()->findByPk($name);
	}

	/**
	 * 判断某项配置是否存在
	 * @param string $name 配置项名称
	 */
	protected function exist($name)
	{
		return ConfigRecord::model()->exists('`name`=:name', array(':name'=>$name));
	}
}
/**
 * Congfig 数据库模型
 * @author dong
 */
class ConfigRecord extends CActiveRecord
{
	/**
	 * @return TaxonomyTerm
	 */
	public static function model($model = __CLASS__)
	{
		return parent::model($model);
	}

	/**
	 * 从Config中设置表名
	 */
	public function tableName()
	{
		return Yii::app()->config->table;
	}
	
	public function primaryKey()
	{
		return 'name';
	}
	
	public function rules()
	{
		return array(
			array('name', 'match', 'pattern'=>'/^[a-zA-Z0\._-]+$/'),
			array('name, value', 'safe'),
		);
	}
	
	/**
	 * 保存之前将value的值序列化
	 * @return boolean
	 */
	public function beforeSave()
	{
		if (parent::beforeSave()) {
			$this->value = serialize($this->value);
			return true;
		}
		return false;
	}
	
	/**
	 * 读取记录之后，恢复里面存储的值为原始格式
	 */
	public function afterFind()
	{
		$this->value = unserialize($this->value);
	}
}